Support for Prolog by Gertjan van Noord, vannoord@let.rug.nl.

List of supported Prologs:
--------------------------

SICStus (2.1.#8, 2.1.#9, 3 #3, 3 #5)
Quintus (believed)
SWI (1.8.9, 2.8.6)
BinProlog (4.00, 5.75)
B-Prolog 2.0

List of Prologs with known problems:
------------------------------------

<none so far>


Check the following:
--------------------

* Built-in predicates used in generated code

see/1, seen/0, get0/1, !/0, =/2, =</2, >=/2, =:=/2  ;/2  ->/2.

if your Prolog doesn't have these, then your Prolog isn't Prolog.

* further (very useful) built-in predicates used in examples

number_chars/2, atom_chars/2, format/3

supported at least in: SICStus, Quintus, SWI.

BinProlog only has atom_chars (with a different semantics). Moreover,
You might have to replace both number_chars and atom_chars with name/2.
BinProlog is very sensitive to the number of atoms. E.g. tokenizing large
text files while constructing atoms for each word will quickly lead to
out of memory problems, which can only be solved at the cost of starting
BinProlog with extraordinary large values of the -a option.

For B-Prolog, use atom_codes and number_codes (no; the latter is in the
manual, but doesn't work).

Instead of format, use a number of write/2 statements.

* Special treatment of character codes:

- end of file: -1
- end of line: 10

* Module support. If your Prolog doesn't support modules, then recompile 
  Prolog code generator without the -DUSE_MODULE switch (or alternatively
  ignore the module declaration at the beginning of the generated files).



Options
-------

check the elex.cfg file for options. There are currently three options
to the Prolog generator (except for the relevant files):

- option 4: 0 or 1 (0: produces fast but verbose code
                    1: produces slower but compact code
	           )

- option 5: 0 or 1 (0: don't have a module declaration at the top
                    1: have a module declaration
                   )

- option 6: integer indicating the number of tokens of look-ahead.


Generated code
--------------

the generator generates a Prolog file which includes a number of 
useful predicates (cf. the examples). Most notably, it provides
the relation

tokenize(Chars,Tokens)

where Chars is a list of character codes ("a string"), and Tokens is 
the list of tokens. The construction of this list is done as part of 
the body of the Prolog code fragments.

Special variables in Prolog code fragments
------------------------------------------

In the prolog fragments, the variables __Token, __Tokens0, and __Tokens
are used. Here, __Token is the list of character codes of the token
just being recognized. __Tokens0 and __Tokens together represent
in diff-list format the resulting list of tokens. 

Note that each code fragment will be the body of a Prolog clause. Don't
forget to end such a fragment with a dot.

A typical example upon recognizing, say, a Prolog atom, is to
construct an atom from the list of character codes, and then
put that atom in the resulting list of tokens, wrapped inside a
functor which indicates the token, as follows:

 <
 atom_chars(Atom,__Token),
 __Tokens0=[atom(Atom)|__Tokens].
 >

Ignoring certain tokens then is easy too:
 <
 __Tokens0=__Tokens.
 >


In the error production there are three different special variables:
__Char, __Chars, and __Tokens.

error is called such that the first argument is the next
character, the second is the rest of the characters, 
and third argument should be the resulting list of tokens.

i.e. the code fragment of the error rule will become the body of
a clause with the head: 
% error(__Char,__Chars,__Tokens) :-


cf. example.

Lookahead
---------

The lookahead feature is only defined for "Pure" prolog code segments.
It's rather inefficient too.

