# $Id: Prompts.pm,v 1.2 1997/04/18 13:16:19 matt Exp $
#
# Routines for displaying interactive prompts for input.

package Prompts;

# set prompts_quiet to 1 if prompts with defaults should return the
# default value instead of reading input from user.
$quiet = 0;

# display a menu for options and returns the option selected.  if
# default is not '', then it is the default option.
sub menu
{
  my ($prompt, $default, @options) = @_;
  my $opt_no, $array_ref;

  # turn default into an array ref
  $array_ref = -1;
  for (0..$#options)
  {
    $array_ref = $_ + 1, last if $options[$_] eq $default;
  }

  # handle case where default is not in options
  $default = '' if $array_ref == -1;

  print ("$prompt:\n\n");

  $opt_no = 1;
  for (@options)
  {
    print ("  ($opt_no) $_\n");
    $opt_no++;
  }

  print ("\n");

  do
  {
    if ($default ne '')
    {
      print ("Choice [$array_ref]: ");
    } else
    {
      print ("Choice: ");
    }

    if ($quiet && $default)
    {
      print ($array_ref, "\n");
      $_ = $array_ref;
    } else
    {
      $_ = <STDIN>;
      $_ = $array_ref if /^\s*$/;
    }

  } while ($_ < 1 || $_ > $#options+1);

  return $options[$_ - 1];
}

# display prompt and read a value.  if default is not '', it is
# returned if no value is entered.
sub get
{
  my ($prompt, $default) = @_;

  do
  {
    if ($default ne '')
    {
      print ("$prompt [$default]: ");
    } else
    {
      print ("$prompt: ");
    }

    if ($quiet && $default)
    {
      print ($default, "\n");
      $_ = $default;
    } else
    {
      $_ = <STDIN>;
      chop;
      $_ = $default if $_ =~ /^\s*$/; 
    }    
  } while ($_ eq '');

  return $_;
}

# get input constrained to a list of choices.  if default is not '',
# then it is returned if no input is made.
sub choice
{
  my ($prompt, $default, @choices) = @_;
  my %choices;

  for (@choices)
  {
    $choices{$_} = 1;
  }

  # handle case of invalid default
  $default = '' if !$choices{$default};

  do
  {
    print ("$prompt (", join ('/', @choices), ")");
    if ($default ne '')
    {
      print (" [$default]? ");
    } else
    {
      print ("? ");
    }

    if ($quiet && $default)
    {
      print ($default, "\n");
      $_ = $default;
    } else
    {
      $_ = <STDIN>;
      chop;
      $_ = $default if $_ =~ /^\s*$/;      
    }
  } while ($_ eq '' || !defined ($choices{$_}));

  return $_;
}

# uses get_choice to get a "y/n" choice.
sub yesNo
{
  my ($prompt, $default) = @_;

  return choice ($prompt, $default, ('y', 'n'));
}

1;
